/* 
   neon interop test suite
   Copyright (C) 2001, Joe Orton <joe@manyfish.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "config.h"

#include <sys/types.h>

#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <fcntl.h>

#include <ne_request.h>
#include <ne_string.h>

#include "interop.h"

static char *src, *dest, *coll, *ncoll;

static int copy_ok = 0;

int copy_init(void)
{
    CONCAT2(src, i_path, "copysrc");
    CONCAT2(dest, i_path, "copydest");
    CONCAT2(coll, i_path, "copycoll/");
    CONCAT2(ncoll, i_path, "copycoll");

    ONREQ(upload_foo("copysrc"));
    ONREQ(ne_mkcol(i_session, coll));

    copy_ok = 1;
    return OK;
}

int copy_simple(void)
{
    PRECOND(copy_ok);

    /* Now copy it once */
    ONREQ(ne_copy(i_session, 0, NE_DEPTH_INFINITE, src, dest));

    if (STATUS(201)) {
	t_warning("COPY to new resource didn't give 201");
    }

    return OK;
}

int copy_overwrite(void)
{
    PRECOND(copy_ok);

    /* Do it again with Overwrite: F to check that fails. */
    ONN("COPY on existing resource with Overwrite: F",
	ne_copy(i_session, 0, NE_DEPTH_INFINITE, src, dest) != NE_ERROR);

    if (STATUS(412)) {
	t_warning("COPY-on-existing fails with 412");
    }
    
    ONN("COPY-on-existing with Overwrite: T",
       ne_copy(i_session, 1, NE_DEPTH_INFINITE, src, dest));

    /* tricky one this, I didn't think it should work, but the spec
     * makes it look like it should. */
    ONN("COPY overwrites collection", 
	ne_copy(i_session, 1, NE_DEPTH_INFINITE, src, ncoll));
    
    if (STATUS(204)) {
	t_warning("COPY to existing resource didn't give 204");
    }

    return OK;
}

int copy_cleanup(void)
{
    ne_delete(i_session, src);
    ne_delete(i_session, dest);
    ne_delete(i_session, ncoll);
    ne_delete(i_session, coll);
    return OK;
}

int copy_coll(void)
{
    int n;
    char *csrc, *cdest, *rsrc, *rdest, *subsrc, *subdest, *cdest2;
    char res[512];

    CONCAT2(csrc, i_path, "ccsrc/");
    CONCAT2(cdest, i_path, "ccdest/");
    CONCAT2(cdest2, i_path, "ccdest2/");
    CONCAT2(rsrc, i_path, "ccsrc/foo");
    CONCAT2(rdest, i_path, "ccdest/foo"); 
    CONCAT2(subsrc, i_path, "ccsrc/subcoll/");
    CONCAT2(subdest, i_path, "ccdest/subcoll/");

    /* Set up the ccsrc collection. */
    ne_delete(i_session, csrc); /* clear up left-overs */
    ONREQ(ne_mkcol(i_session, csrc));
    for (n = 0; n < 10; n++) {
	sprintf(res, "ccsrc/foo.%d", n);
	ONREQ(upload_foo(res));
    }
    ONREQ(ne_mkcol(i_session, subsrc));
    
    /* Clean up to make some fresh copies. */
    ne_delete(i_session, cdest);
    ne_delete(i_session, cdest2);

    /* Now copy the collection a couple of times */
    ONREQ(ne_copy(i_session, 0, NE_DEPTH_INFINITE, csrc, cdest));
    ONREQ(ne_copy(i_session, 0, NE_DEPTH_INFINITE, csrc, cdest2));

    ONN("COPY-on-existing-coll should fail",
	ne_copy(i_session, 0, NE_DEPTH_INFINITE, cdest, cdest2) != NE_ERROR);
    
    ONN("COPY-on-existing-coll with overwrite",
	ne_copy(i_session, 1, NE_DEPTH_INFINITE, cdest2, cdest));

    /* Remove the source, to be paranoid. */
    if (ne_delete(i_session, csrc)) {
	t_warning("Could not delete csrc");
    }

    /* Now delete things out of the destination collection to check if
     * they are there. */
    for (n = 0; n < 10; n++) {
	sprintf(res, "%s%s.%d", i_path, "ccdest/foo", n);
	ONREQ(ne_delete(i_session, res));
    }
    ONREQ(ne_delete(i_session, subdest));

    /* Now nuke the whole of the second copy. */
    ONREQ(ne_delete(i_session, cdest2));

    if (ne_delete(i_session, cdest)) {
	t_warning("Could not clean up cdest");
    }

    return OK;
}

int move(void)
{
    int fd = open("htdocs/foo", O_RDONLY);
    char *src2;

    CONCAT2(src, i_path, "move");
    CONCAT2(src2, i_path, "move2");
    CONCAT2(dest, i_path, "movedest");
    CONCAT2(coll, i_path, "movecoll/");
    CONCAT2(ncoll, i_path, "movecoll");

    ONN("could not open htdocs/foo", fd < 0);

    /* Upload it twice. */
    ONREQ(ne_put(i_session, src, fd));
    ONREQ(ne_put(i_session, src2, fd));
    ONREQ(ne_mkcol(i_session, coll));

    close(fd);

    /* Now move it */
    ONREQ(ne_move(i_session, 0, src, dest));

    if (STATUS(201)) {
	t_warning("MOVE to new resource didn't give 201");
    }

    /* Try a move with Overwrite: F to check that fails. */
    ONN("MOVE on existing resource with Overwrite: F",
	ne_move(i_session, 0, src2, dest) != NE_ERROR);

    if (STATUS(412)) {
	t_warning("MOVE-on-existing should fail with 412");
    }

    ONN("MOVE onto existing resource with Overwrite: T",
	ne_move(i_session, 1, src2, dest));

    ONN("MOVE overwrites collection", 
	ne_move(i_session, 1, dest, ncoll));
    
    if (STATUS(204)) {
	t_warning("MOVE to existing resource didn't give 204");
    }
		  
    if (ne_delete(i_session, ncoll)) {
	t_warning("Could not clean up movecoll");
    }

    return OK;
}

static char *mdest, *msrc, *mdest2, *mnoncoll;

int move_coll(void)
{
    int n;
    char *rsrc, *rdest, *subsrc, *subdest;
    char res[512];

    CONCAT2(msrc, i_path, "mvsrc/");
    CONCAT2(mdest, i_path, "mvdest/");
    CONCAT2(mdest2, i_path, "mvdest2/");
    CONCAT2(rsrc, i_path, "mvsrc/foo");
    CONCAT2(rdest, i_path, "mvdest/foo"); 
    CONCAT2(subsrc, i_path, "mvsrc/subcoll/");
    CONCAT2(subdest, i_path, "mvdest/subcoll/");
    CONCAT2(mnoncoll, i_path, "mvnoncoll");

    /* Set up the mvsrc collection. */
    ONREQ(ne_mkcol(i_session, msrc));
    for (n = 0; n < 10; n++) {
	sprintf(res, "mvsrc/foo.%d", n);
	ONREQ(upload_foo(res));
    }
    ONREQ(upload_foo("mvnoncoll"));
    ONREQ(ne_mkcol(i_session, subsrc));
    
    /* Now make a copy of the collection */
    ONREQ(ne_copy(i_session, 0, NE_DEPTH_INFINITE, msrc, mdest2));
    ONREQ(ne_move(i_session, 0, msrc, mdest));

    ONN("MOVE-on-existing-coll should fail",
	ne_move(i_session, 0, mdest, mdest2) != NE_ERROR);
    
    ONN("MOVE-on-existing-coll with overwrite",
	ne_move(i_session, 1, mdest2, mdest));

    /* Take another copy. */
    ONREQ(ne_copy(i_session, 0, NE_DEPTH_INFINITE, mdest, mdest2));

    /* Now delete things out of the destination collection to check if
     * they are there. */
    for (n = 0; n < 10; n++) {
	sprintf(res, "%s%s.%d", i_path, "mvdest/foo", n);
	ONREQ(ne_delete(i_session, res));
    }
    ONREQ(ne_delete(i_session, subdest));

    /* And move the spare collection over a non-coll. */
    ONN("MOVE collection over non-collection with overwrite",
	ne_move(i_session, 1, mdest2, mnoncoll));

    return OK;
}

int move_cleanup(void)
{
    ne_delete(i_session, mdest);
    ne_delete(i_session, mdest2);
    ne_delete(i_session, mnoncoll);
    return OK;
}
